// Argussoft SSC library

#include "argussoft.h"
#include "cdc_enumerate.h"


AT91PS_SSC SSC = AT91C_BASE_SSC;


#define   SAMPLEWORD                    unsigned short
#define   SAMPLEWORDS                   1
#define   SAMPLEWORDBITS                (sizeof(SAMPLEWORD) * 8)
#define   SAMPLEFRAME                   8
#define   SAMPLE_RATE                   10000


#define AT91C_SSC_TD         AT91C_PA17_TD
#define AT91C_SSC_TF         AT91C_PA15_TF
#define AT91C_SSC_TK         AT91C_PA16_TK
#define AT91C_SSC_RD         AT91C_PA18_RD
#define AT91C_SSC_RF         AT91C_PA20_RF
#define AT91C_SSC_RK         AT91C_PA19_RK


//#define SSCC    (1<<16) // pa16 SSC clock
//#define SSCF    (1<<15) // pa15 SSC clock
//#define SSCDOUT (1<<17) // pa17 SSC data out
//#define SSCDIN  (1<<18) // pa18 SSC data in


unsigned int SSC_GetStatus(void)
{
  return(SSC->SSC_SR);
};

unsigned int SSC_Transfer(unsigned int data)
{
  //A while statement executes a statement zero or more times, while the test-context expression has a nonzero value:

  // wait TXRDY signal will be set, SSC_THR will be ready to writing
  // !!! check this point !!!

  //while(!(SSC->SSC_SR & AT91C_SSC_TXRDY)){};
  SSC->SSC_THR = data;

  // wait RXRDY signal will be set, SSC_RHR will be ready to reading
  // !!! check this point !!!
  //while(!(SSC->SSC_SR & AT91C_SSC_RXRDY)){};
  return(SSC->SSC_RHR);
};

void SSC_SendWord(unsigned int data)
{
  SSC->SSC_THR = data;
};


unsigned int SSC_GetWord(void)
{
  return(SSC->SSC_RHR);
};


//*----------------------------------------------------------------------------
//* \fn    AT91F_SSC_Conf
//* \brief Configure SSC
//*----------------------------------------------------------------------------
__inline void AT91F_SSC_Conf (
             AT91PS_SSC pSSC,          // \arg pointer to a SSC controller
             unsigned int cmr,         // \arg System Clock Frequency
             unsigned int clock_rx,    // \arg Receiver Clock Parameters
             unsigned int mode_rx,     // \arg mode Register to be programmed
             unsigned int clock_tx,    // \arg Transmitter Clock Parameters
             unsigned int mode_tx)     // \arg mode Register to be programmed
{    //* Disable interrupts
	pSSC->SSC_IDR = (unsigned int) -1;

    //* Reset receiver and transmitter
	pSSC->SSC_CR = AT91C_SSC_SWRST | AT91C_SSC_RXDIS | AT91C_SSC_TXDIS ;

    //* Define the Clock Mode Register
        pSSC->SSC_CMR = cmr;

     //* Write the Receive Clock Mode Register
	pSSC->SSC_RCMR =  clock_rx;

     //* Write the Transmit Clock Mode Register
	pSSC->SSC_TCMR =  clock_tx;

     //* Write the Receive Frame Mode Register
	pSSC->SSC_RFMR =  mode_rx;

     //* Write the Transmit Frame Mode Register
	pSSC->SSC_TFMR =  mode_tx;

    //* Clear Transmit and Receive Counters
	//AT91F_PDC_Open((AT91PS_PDC) &(pSSC->SSC_RPR));
}

//  SSC - 

void SSC_Init(void)
{
  //  Signal Symbol PIO controller Description
  //  RK (AT91C_PA19_RK ) PIOA Periph: A Bit: 19 SSC Receive Clock
  //  TK (AT91C_PA16_TK ) PIOA Periph: A Bit: 16 SSC Transmit Clock
  //  TF (AT91C_PA15_TF ) PIOA Periph: A Bit: 15 SSC Transmit Frame Sync
  //  RD (AT91C_PA18_RD ) PIOA Periph: A Bit: 18 SSC Receive Data
  //  RF (AT91C_PA20_RF ) PIOA Periph: A Bit: 20 SSC Receive Frame Sync
  //  TD (AT91C_PA17_TD ) PIOA Periph: A Bit: 17 SSC Transmit data

  // enable clock on SSC
  AT91F_SSC_CfgPMC();
  // don`t call function AT91F_SSC_CfgPIO because, we don`t need all 6 pins, only 4 of them

  AT91F_SSC_CfgPIO();
/*
  AT91F_PIO_CfgPeriph(
                AT91C_BASE_PIOA, // PIO controller base address
                ((unsigned int) AT91C_PA16_TK) |
                ((unsigned int) AT91C_PA15_TF) |
                ((unsigned int) AT91C_PA18_RD) |
                ((unsigned int) AT91C_PA20_RF) |
                ((unsigned int) AT91C_PA17_TD), // Peripheral A
                0); // Peripheral B
*/
/*
   // pio Special configuration
      *AT91C_PIOA_PDR             = AT91C_SSC_TD;
      *AT91C_PIOA_ASR             = AT91C_SSC_TD;
      *AT91C_PIOA_ODR             = AT91C_SSC_TD;


      *AT91C_PIOA_PDR             = AT91C_SSC_TF;
      *AT91C_PIOA_ASR             = AT91C_SSC_TF;
      *AT91C_PIOA_ODR             = AT91C_SSC_TF;

      *AT91C_PIOA_PDR             = AT91C_SSC_TK;
      *AT91C_PIOA_ASR             = AT91C_SSC_TK;
      *AT91C_PIOA_ODR             = AT91C_SSC_TK;
*/


/*
  // Disable interrupts
  SSC->SSC_IDR = (unsigned int) -1;

  // Reset receiver and transmitter
  SSC->SSC_CR = AT91C_SSC_SWRST | AT91C_SSC_RXDIS | AT91C_SSC_TXDIS ;
*/

/*
  // configure registers
  AT91F_SSC_Configure(
                        SSC,
                        48000000, //  SSC_CMR register, clock rate for SSC interface
                        1000000,  // ^^^^^^^
                        0x00000001,// SSC_RCMR register
                        0x20000208,// SSC_TCMR register,
                        0x0000008f,// SSC_RFMR register
                        0x0030008f// SSC_TFMR register
                        );
*/
/*
  // configure registers
  AT91F_SSC_Configure(
                        SSC,
                        48000000, //  SSC_CMR register, clock rate for SSC interface
                        1000000,  // ^^^^^^^
                        0x00000000,// SSC_RCMR register
                        0x00000124,// SSC_TCMR register,
                        0x00000000,// SSC_RFMR register
                        0x0030008f// SSC_TFMR register
                        );
  */
   // Configure SSC in Transmit mode
      AT91F_SSC_Conf(AT91C_BASE_SSC,
        (MCK / (2 * SAMPLEWORDBITS * SAMPLE_RATE)) + 1,             // SSC_CMR


        (AT91C_SSC_CKS_TK + AT91C_SSC_START_FALL_RF + AT91C_SSC_CKO_NONE),   // SSC_RCMR
        (SAMPLEWORDBITS - 1) + AT91C_SSC_MSBF + AT91C_SSC_FSOS_NONE, // SSC_RFMR

        (AT91C_SSC_CKS_DIV + AT91C_SSC_CKO_CONTINOUS + AT91C_SSC_START_CONTINOUS + AT91C_SSC_CKI), // SSC_TCMR
        (SAMPLEWORDBITS - 1) + AT91C_SSC_MSBF + AT91C_SSC_FSOS_LOW); // SSC_TFMR



  // enable tx and rx operations
//  AT91F_SSC_EnableRx(SSC);
//  AT91F_SSC_EnableTx(SSC);
       AT91F_SSC_EnableTx(AT91C_BASE_SSC);
       AT91F_SSC_EnableRx(AT91C_BASE_SSC);
};



